unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, xmldom, XMLIntf, Spin, msxmldom, XMLDoc, Sockets,
  ComCtrls, TabNotBk;


type
  TTrain=record
    name:string;
    stationCount:integer;
    stationList:array of string;
    stationTimes:array of integer;
    noteCount: integer;
    noteList: array of string;
  end;

type
  TServerForm = class(TForm)
    TabbedNotebook1: TTabbedNotebook;
    Label1: TLabel;
    DataFile: TEdit;
    Debug: TListBox;
    TcpServer: TTcpServer;
    XMLDoc: TXMLDocument;
    LoadFile: TButton;
    Label2: TLabel;
    Label3: TLabel;
    SpinEdit1: TSpinEdit;
    Run: TButton;
    OpenDialog: TOpenDialog;
    CityList: TListBox;
    TrainList: TListBox;
    procedure LoadFileClick(Sender: TObject);
    procedure RunClick(Sender: TObject);
  private
    Trains: array of TTrain;
    procedure DecodeTrain(node: IXMLNode; number: integer);
    { Private declarations }
  public
    { Public declarations }
  end;

var
  ServerForm: TServerForm;

implementation

{
/*********************************************************************
 Server

 myslenka -- opet bezmyslenkovita implementace parseru (nacitani dat)
 a tcp/ip komunikace. Krome toho ale taky algoritmus -- pouzijeme
 pozmeneneho dijkstrova, kde vzdalenost bodu udava cas potrebny pro
 presun mezi nimi (vcetne cekani na nejblizsi vlak). Datova struktura
 je nasledovna:

 pole vsech stanic (pro pozadavek seznamstanic bude treba abecedne)

 pro kazdou stanici uchovavame casy, serazene od rana do vecera, kdy
 do nich prichazi vlak a pointer na tento vlak

 dale seznam vsech vlaku, u kazdeho seznam stanic, kterymi projizdi

 V teto strukture dokazeme v case O(pocet stanic vlaku) zjistit,
 jestli danou stanici vlak prochazi. V case O(pocet stanic vlaku *
 pocet vlaku ktere projdou stanici za den) dokazeme najit nejrychlejsi
 primy spoj z jedne stanice do druhe. Kdyz dokazeme najit nejrychlejsi
 cestu mezi dvemi stanicemi, muzeme vystavet graf, na ktery aplikujeme
 uz djikstru.
*********************************************************************/
}


{$R *.dfm}

// pro univerzalni porovnavani casu
// UCF -- Univerzalni casovy format

const tDay = 60*24;   // toto cislo lze libovolne pricitat a odcitat od UCF

function DecodeTime(time:string): integer;
var
  p: integer;
  hod, min: string;
begin
  p := pos(':', time);
  hod := copy(time, 1, p-1);
  min := copy(time, p+1, 2);
  Result := StrToInt(hod)*60+StrToInt(min);
end;

function EncodeTime(time:integer): string;
begin
  time := time mod tDay;
  Result := IntToStr(time div 60) + ':' + IntToStr(time mod 60);
end;


procedure TServerForm.DecodeTrain(node: IXMLNode; number: integer);
var
  i: integer;
begin
  // at first, just count all that stanice/poznamka nodes
  Trains[number].stationCount := 0;
  Trains[number].noteCount := 0;

  TrainList.Items.Add(node.Text);
  Trains[number].name := node.Text;

  for i:=0 to node.ChildNodes.Count-1 do begin
    if (node.ChildNodes.Get(i).NodeName = 'Stanice') then Inc(Trains[number].stationCount)
    else if (node.ChildNodes.Get(i).NodeName = 'Poznamka') then Inc(Trains[number].noteCount);
  end;

  // now, let's fill that arrays
  setlength(Trains[number].stationList, Trains[number].stationCount);
  setlength(Trains[number].noteList, Trains[number].noteCount);
end;

procedure TServerForm.LoadFileClick(Sender: TObject);
begin
  if (Opendialog.Execute) then begin
    DataFile.Text := OpenDialog.Files[0];
  end;
end;

procedure TServerForm.RunClick(Sender: TObject);
var
  node: IXMLNode;
  i, count: integer;
begin
  // parse all the data

  XMLDoc.LoadFromFile(DataFile.Text);
  Debug.Items.Add('XML file loaded.'); Debug.Repaint;

  node := XMLDoc.ChildNodes.FindNode('Linky');

  count := 0;
  for i:= 0 to node.ChildNodes.Count-1 do begin
    if (node.ChildNodes.Get(i).NodeName = 'Linka') then Inc(Count);
  end;

  setlength(Trains, count);

  for i:= 0 to node.ChildNodes.Count-1 do begin
    if (node.ChildNodes.Get(i).NodeName = 'Linka') then DecodeTrain(node.ChildNodes.Get(i), i);
  end;

end;

end.
